import math
from datetime import datetime
from io import BytesIO
from typing import BinaryIO, Sequence

from cap.core import (
    CAPTURED_PACKET_HEADER_STRUCTURE,
    NETWORK_CAPTURE_HEADER_STRUCTURE,
    PCAP_MAGICS,
    CapturedPacket,
    Endianness,
    LinkLayerTypes,
    SecondsPartsUnit,
)


def load(stream: BinaryIO):
    endianness, seconds_parts_type = PCAP_MAGICS[stream.read(4)]

    (
        _,
        _,
        time_zone_offset_hours,
        _,
        max_capture_length_octets,
        link_layer_type,
    ) = NETWORK_CAPTURE_HEADER_STRUCTURE[
        endianness
    ].unpack(stream.read(20))

    captured_packet_header_structure = CAPTURED_PACKET_HEADER_STRUCTURE[endianness]

    while True:
        packet_header_bytes = stream.read(16)
        if not packet_header_bytes:
            break
        packet_header_values = captured_packet_header_structure.unpack(packet_header_bytes)
        seconds, seconds_parts, data_length, original_length = packet_header_values

        yield CapturedPacket(
            data=stream.read(data_length),
            capture_time=datetime.fromtimestamp(
                (float(seconds) + (seconds_parts_type * seconds_parts)) - time_zone_offset_hours
            ),
            original_length=original_length,
            link_layer_type=link_layer_type,
        )


def loads(data: bytes):
    return load(BytesIO(data))


def dump(
    captured_packets: Sequence[CapturedPacket],
    stream: BinaryIO,
    write_header=True,
    endianness: Endianness = Endianness.little_endian,
    seconds_parts_unit: SecondsPartsUnit = SecondsPartsUnit.micros,
    major_version=2,
    minor_version=4,
    time_zone_offset_hours: int = 0,
    max_capture_length_octets: int = 0x40000,
    link_layer_type: LinkLayerTypes = LinkLayerTypes.ethernet,
):
    if write_header:
        stream.write(PCAP_MAGICS.inverse[(endianness, seconds_parts_unit)])
        stream.write(
            NETWORK_CAPTURE_HEADER_STRUCTURE[endianness].pack(
                major_version,
                minor_version,
                time_zone_offset_hours,
                0,
                max_capture_length_octets,
                link_layer_type,
            )
        )
    for captured_packet in captured_packets:
        seconds, seconds_parts = math.modf(captured_packet.capture_time.timestamp() + time_zone_offset_hours)

        stream.write(
            CAPTURED_PACKET_HEADER_STRUCTURE[endianness].pack(
                seconds,
                math.trunc(seconds_parts / seconds_parts_unit),
                len(captured_packet.data),
                captured_packet.original_length,
            )
        )


def dumps(captured_packets: Sequence[CapturedPacket]):
    stream = BytesIO()
    dump(captured_packets, stream)
    return str(stream)


def merge(target_stream, *source_streams):
    result = []
    for source_stream in source_streams:
        result.extend(load(source_stream))
    return dump(result, target_stream)



from dataclasses import dataclass
from datetime import datetime
from enum import Enum, IntEnum, StrEnum, auto
from struct import Struct

from bidict import bidict


class LinkLayerTypes(IntEnum):
    none = 0
    ethernet = auto()


class Endianness(StrEnum):
    big_endian = ">"
    little_endian = "<"


class SecondsPartsUnit(float, Enum):
    micros = 1e6
    nanos = 1e9


PCAP_MAGICS = bidict(
    [
        (b"\xd4\xc3\xb2\xa1", (Endianness.little_endian, SecondsPartsUnit.micros)),
        (b"\xd4\x3c\xb2\xa1", (Endianness.little_endian, SecondsPartsUnit.nanos)),
        (b"\xa1\xb2\xc3\xd4", (Endianness.big_endian, SecondsPartsUnit.micros)),
        (b"\xa1\xb2\x3c\xd4", (Endianness.big_endian, SecondsPartsUnit.nanos)),
    ]
)

NETWORK_CAPTURE_HEADER_STRUCTURE = {
    Endianness.big_endian: Struct(f"{Endianness.big_endian}HHiIII"),
    Endianness.little_endian: Struct(f"{Endianness.little_endian}HHiIII"),
}

CAPTURED_PACKET_HEADER_STRUCTURE = {
    Endianness.big_endian: Struct(f"{Endianness.big_endian}IIII"),
    Endianness.little_endian: Struct(f"{Endianness.little_endian}IIII"),
}


@dataclass
class CapturedPacket:
    data: bytes
    capture_time: datetime
    link_layer_type: LinkLayerTypes = LinkLayerTypes.ethernet
    original_length: int | None = None

    def __post_init__(self):
        self.original_length = self.original_length or len(self.data)

    @property
    def is_fully_captured(self):
        return self.original_length == len(self.data)

    def __iter__(self):
        return iter(self.data)

    def __getitem__(self, item):
        return self.data[item]

    def __repr__(self):
        return f"<CapturedPacket - {len(self.data)} bytes captured at {self.capture_time} >"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['cap']

package_data = \
{'': ['*']}

install_requires = \
['bidict>=0.22.1,<0.23.0']

setup_kwargs = {
    'name': 'cap',
    'version': '0.1.2',
    'description': 'Cap: lightweight package for use network captures',
    'long_description': '# Cap: lightweight package for use network captures\n\n[![Build Status](https://travis-ci.org/netanelrevah/cap.svg?branch=develop)](https://travis-ci.org/netanelrevah/cap) [![PyPI version](https://img.shields.io/pypi/v/cap.svg)](https://pypi.python.org/pypi/cap/) [![PyPI downloads](https://img.shields.io/pypi/dm/cap.svg)](https://pypi.python.org/pypi/cap/)\n\nThe idea is to read and write capture files like it is really a serialized data. The API is ment to be close as possible to json and pickle APIs.\n\n## Installation:\ninstall the package by:\n``` bash\npip install cap\n```\nor from the source:\n``` bash\npython setup.py install\n```\n## Usage:\n### read cap:\n```python\nimport cap\ncaptured_packets = cap.load(open("C:\\\\test.cap", "rb"))\n```\n### filter about somthing\n```python\nip_v4_captured_packet = []\nfor captured_packet in captured_packets:\n    if captured_packet.data[12:14] == \'\\x08\\x00\':\n        ip_v4_captured_packet.append(p)\n```\n### dump filtered packets\n```python\ncap.dump(ip_v4_captured_packet, open(\'C:\\\\new_test.cap\', "wb"))\n```\n\nHave a nice use and please report about problems and issues.\nThank you.\n',
    'author': 'Netanel Revah',
    'author_email': 'netanelrevah@outlook.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
